unit WABD_EditTable;

interface

uses SysUtils, Classes, WABD_Objects;

type
   // Derive from TWABD_FormSection_Base insead of just TWABD_FormSection
   // so that the user can NOT edit the child controls in the WABD Form
   // designer
   TWABD_EditTable = class(TWABD_FormSection_Base)
   protected
      FCols       : integer;
      FRows       : integer;
      FEdits      : TList;
      FEditSize   : integer;
      function    GetCell(x, y: integer): string;
      procedure   SetCell(x, y: integer; const Value: string);
      procedure   SetCols(NewCols: integer);
      procedure   SetRows(NewRows: integer);
      procedure   Resize;
   public
      constructor Create(AOwner: TComponent); override;
      destructor  Destroy; override;
      property    Cells[x,y: integer]: string read GetCell write SetCell;
   published
      property    Cols: integer read FCols write SetCols;
      property    Rows: integer read FRows write SetRows;
      property    GridX;
      property    GridY;
      property    EditSize: integer read FEditSize write FEditSize;
   end;

   procedure Register;

implementation


constructor TWABD_EditTable.Create(AOwner: TComponent);
begin
   inherited;
   FEdits     := TList.Create;
   GridX      := 60;
   GridY      := 25;
   FCols      := 5;
   FRows      := 5;
   CellBorder := 0;
   FEditSize  := 7;
   Resize;
end;

destructor TWABD_EditTable.Destroy;
begin
   FEdits.Free;
   inherited;
end;

procedure TWABD_EditTable.SetCols(NewCols: integer);
begin
   FCols := NewCols;
   Resize;
end;

procedure TWABD_EditTable.SetRows(NewRows: integer);
begin
   FRows := NewRows;
   Resize;
end;

procedure TWABD_EditTable.Resize;
var
   i, x, y : integer;
   ne      : TWABD_Edit;
begin
   for i := FEdits.Count-1 downto 0 do begin
      TObject(FEdits[i]).Free;
      FEdits.Delete(i);
   end;

   for x := 0 to FCols-1 do begin
      for y := 0 to FRows-1 do begin
         ne := TWABD_Edit.Create(Self);
         ne.Parent  := Self;
         ne.Name    := Format('%s_x%d_y%d', [Name, x, y]);
         ne.LeftPos := x * GridX;
         ne.TopPos  := y * GridY;
         ne.Size    := FEditSize;
         ne.Text    := '';
         FEdits.Add(ne);
      end;
   end;
end;

function TWABD_EditTable.GetCell(x, y: integer): string;
var
   ne : TWABD_Edit;
begin
   ne := FEdits.Items[x * FRows + y];
   Result := ne.Text;
end;

procedure TWABD_EditTable.SetCell(x, y: integer; const Value: string);
var
   ne : TWABD_Edit;
begin
   ne := FEdits.Items[x * FRows + y];
   ne.Text := Value;
end;


procedure Register;
begin
   RegisterNoIcon([TWABD_EditTable]);
end;


initialization
   RegisterWABDComponents([TWABD_EditTable]);
   RegisterClasses([TWABD_EditTable]);
end.
