unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, WinSock, Menus, TrayIcon, ClipBrd, ComCtrls, BenTools;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ServerName: TLabel;
    ServerAddress: TLabel;
    Label3: TLabel;
    NumHits: TLabel;
    Label4: TLabel;
    ErrorLab: TLabel;
    TrayIcon1: TTrayIcon;
    PopupMenu1: TPopupMenu;
    WWWServer1: TMenuItem;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Close1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    Shutdown1: TMenuItem;
    Tools1: TMenuItem;
    ClearLog1: TMenuItem;
    Label5: TLabel;
    PortLab: TLabel;
    CopyLogtoClipboard1: TMenuItem;
    ListView1: TListView;
    HitIcon: TImage;
    NormalIcon: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure WWWServer1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure Exit1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure ClearLog1Click(Sender: TObject);
    procedure CopyLogtoClipboard1Click(Sender: TObject);
    procedure ServerAddressDblClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListView1DblClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    LisSock    : TSocket;
    Hits       : integer;
    procedure  Start_Server;
    procedure  Stop_Server;
    procedure  UpdateHits;
    procedure  Log(Hit: integer; const RemoteHost, Msg, Data, Hdrs, ErrMsg: string);
    procedure  LogToFile(Hit: integer; const RemoteHost, Msg, Data, Hdrs, ErrMsg: string);
  end;

var
  MainForm: TMainForm;

implementation

uses ListenThread, HitDetail;

{$R *.DFM}

procedure TMainForm.Start_Server;
const
   CR       = #13#10;
   PORT     = 80;                    // Real WWW servers are on 80!
var
   wd       : TWSAdata;
   rc       : integer;
   sa       : TSockAddr;
   buf      : array[0..MAX_PATH] of char;
   buf2     : PChar;
   in_addr  : PInAddr;
   he       : PHostEnt;
   lt       : TListenThread;
begin
   // Startup WinSock
   rc := WSAStartup($0101, wd);
   Assert(rc=0, 'Unable to Start WinSock');

   LisSock := socket(PF_INET, SOCK_STREAM, 0);

   FillChar(sa, sizeof(sa), 0);
   sa.sin_family      := AF_INET;
   sa.sin_port        := htons(PORT);
   sa.sin_addr.S_addr := INADDR_ANY;
   rc := bind(LisSock, sa, sizeof(sa));
   if rc<>0 then begin
      MessageDlg('FATAL ERROR: Unable to bind to Server Socket!'#13#10'Shutting Down!',
         mtError, [mbOK], 0);
      Application.Terminate;
   end;
   // Assert(rc=0, 'Unable to bind');

   rc := gethostname(buf, sizeof(buf));
   Assert(rc=0, 'Unable to GetHostName');
   he := gethostbyname(buf);
   if he=nil then begin
      MessageDlg('FATAL ERROR:  Unable to GetHostByName'#13 +
      'Be sure to start Dial-Up Networking BEFORE you start the Server',
      mtError, [mbOk], 0);
      Application.Terminate;
      exit;
   end;
   Assert(he<>nil, 'Unable to GetHostByName');
   in_addr := PInAddr(he.h_addr^);
   buf2 := inet_ntoa(in_addr^);

   ServerName.Caption := he.h_name;
   ServerAddress.Caption := buf2;
   PortLab.Caption := IntToStr(PORT);

   // Listen
   rc := listen(LisSock, 10);
   Assert(rc=0, 'Unable to Listen');

   lt := TListenThread.Create(True);
   lt.LisSock := LisSock;
   lt.Resume;
end;


procedure TMainForm.Stop_Server;
var
   rc : integer;
begin
   // Close the Socket
   rc := closesocket(LisSock);
   Assert(rc=0, 'Unable to Close LisSocket');

   // Shutdown WinSock
   rc := WSACleanup;
   Assert(rc=0, 'Unable to Shutdown WinSock');
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
   Hits := -1;
   UpdateHits;
   Start_Server;
   Log(0, '', 'Startup', '', '', '');
   TrayIcon1.Icon := NormalIcon.Picture.Icon;
   Visible := False;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
   Stop_Server;
end;

procedure TMainForm.UpdateHits;
begin
   Hits := Hits + 1;
   NumHits.Caption := Format('%1.0n', [Hits+0.0]);
end;

procedure TMainForm.WWWServer1Click(Sender: TObject);
begin
   Show;
   WindowState := wsNormal;
   BringToFront;
   SetFocus;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   CanClose := False;
   Hide;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
   Application.Terminate;
end;

procedure TMainForm.Close1Click(Sender: TObject);
begin
   Close;
end;

procedure TMainForm.ClearLog1Click(Sender: TObject);
begin
   ListView1.Items.Clear;
end;

procedure TMainForm.LogToFile(Hit: integer; const RemoteHost, Msg, Data, Hdrs, ErrMsg: string);
const
   LOGFILE = 'WWWLOG.TXT';
var
   fs : TFileStream;
   sl : TStringList;
   s  : string;
begin
   try
      fs := TFileStream.Create(LOGFILE, fmOpenReadWrite or fmShareDenyWrite);
   except
      fs := TFileStream.Create(LOGFILE, fmCreate or fmOpenReadWrite or fmShareDenyWrite);
   end;
   fs.Seek(0, 2); // Seek EOF

   sl := TStringList.Create;
   sl.Add(IntToStr(Hit));
   sl.Add(RemoteHost);
   sl.Add(Msg);
   sl.Add(Data);
   sl.Add(Hdrs);
   sl.Add(ErrMsg);
   s := sl.CommaText;
   FindReplace(s, #13#10, '\n');
   s := s + #13#10;
   sl.Free;

   fs.Write(s[1], Length(s));
   fs.Free;
end;

procedure TMainForm.Log(Hit: integer; const RemoteHost, Msg, Data, Hdrs, ErrMsg: string);
var
   li : TListItem;
begin
   LogToFile(Hit, RemoteHost, Msg, Data, Hdrs, ErrMsg);

   li := ListView1.Items.Add;
   li.Caption := FormatDateTime('ddd dd mmm hh:nn:ss', Now);
   li.SubItems.Add(IntToStr(Hit));
   li.SubItems.Add(RemoteHost);
   li.SubItems.Add(Msg);
   li.SubItems.Add(Data);
   li.SubItems.Add(Hdrs);
   li.SubItems.Add(ErrMsg);
   li.MakeVisible(False);

   if ListView1.Items.Count > 5000 then begin
      // ListView1.Items.SaveToFile('TmpWWWLog.txt');
      ListView1.Items.Clear;
   end;

   if not Visible then
      TrayIcon1.Icon := HitIcon.Picture.Icon;
end;

procedure TMainForm.CopyLogtoClipboard1Click(Sender: TObject);
begin
   // Clipboard.SetTextBuf(PChar(ListView1.Items.Text));
end;

procedure TMainForm.ServerAddressDblClick(Sender: TObject);
begin
   ClipBoard.AsText := ServerAddress.Caption;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
   TrayIcon1.Icon := NormalIcon.Picture.Icon;
end;

procedure TMainForm.ListView1DblClick(Sender: TObject);
var
   li : TListItem;
begin
   li := ListView1.Selected;
   if li=nil then exit;

   with HitDetailForm do begin
      LV := ListView1;
      ShowModal;
   end;
end;

end.
